return function(cpu, state, mem, BCD)
  if BCD == nil then
    BCD = true
  end
  local bor, band, bnot, bxor, lshift, rshift
  do
    local _obj_0 = require('bit')
    bor, band, bnot, bxor, lshift, rshift = _obj_0.bor, _obj_0.band, _obj_0.bnot, _obj_0.bxor, _obj_0.lshift, _obj_0.rshift
  end
  local tou8, tos8, tou16
  do
    local tmp = require('ffi').new("\n			struct {\n				uint8_t  u8;\n				int8_t   s8;\n				uint16_t u16;\n			}\n		")
    local u8
    u8 = function(self)
      tmp.u8 = self
      return tmp.u8
    end
    local s8
    s8 = function(self)
      tmp.s8 = self
      return tmp.s8
    end
    local u16
    u16 = function(self)
      tmp.u16 = self
      return tmp.u16
    end
    tou8, tos8, tou16 = u8, s8, u16
  end
  local read8
  do
    local _base_0 = mem
    local _fn_0 = _base_0.read8
    read8 = function(...)
      return _fn_0(_base_0, ...)
    end
  end
  local read16
  do
    local _base_0 = mem
    local _fn_0 = _base_0.read16
    read16 = function(...)
      return _fn_0(_base_0, ...)
    end
  end
  local accum
  accum = function()
    return state.A
  end
  local write8
  do
    local _base_0 = mem
    local _fn_0 = _base_0.write8
    write8 = function(...)
      return _fn_0(_base_0, ...)
    end
  end
  local writea
  writea = function(addr, v)
    state.A = v
  end
  local penalty
  penalty = function(base, addr)
    return band(rshift(bxor(base, addr), 8), 0x01)
  end
  local imp
  imp = function(self) end
  local imm
  imm = function(self)
    local base = self.PC
    self.PC = self.PC + 1
    return base
  end
  local zp
  zp = function(self)
    local base = mem:read8(self.PC)
    self.PC = self.PC + 1
    return base
  end
  local zpx
  zpx = function(self)
    local base = mem:read8(self.PC)
    self.PC = self.PC + 1
    return band(base + self.X, 0xff)
  end
  local zpy
  zpy = function(self)
    local base = mem:read8(self.PC)
    self.PC = self.PC + 1
    return band(base + self.Y, 0xff)
  end
  local ab
  ab = function(self)
    local base = read16(self.PC)
    self.PC = self.PC + 2
    return base
  end
  local abx
  abx = function(self)
    local base = read16(self.PC)
    self.PC = self.PC + 2
    return band(base + self.X, 0xffff)
  end
  local aby
  aby = function(self)
    local base = read16(self.PC)
    self.PC = self.PC + 2
    return band(base + self.Y, 0xffff)
  end
  local rel
  rel = function(self)
    local base = mem:read8(self.PC)
    self.PC = self.PC + 1
    return tos8(base)
  end
  local abxp
  abxp = function(self)
    local base = mem:read16(self.PC)
    self.PC = self.PC + 2
    local addr = band(base + self.X, 0xffff)
    self._cycles = self._cycles + penalty(base, addr)
    return addr
  end
  local abyp
  abyp = function(self)
    local base = mem:read16(self.PC)
    self.PC = self.PC + 2
    local addr = band(base + self.Y, 0xffff)
    self._cycles = self._cycles + penalty(base, addr)
    return addr
  end
  local ind
  ind = function(self)
    local base = mem:read16(self.PC)
    self.PC = self.PC + 2
    return mem:read8(base) + 0x100 * mem:read8(band(base, 0xff00) + band(base + 1, 0x00ff))
  end
  local indx
  indx = function(self)
    local b1 = band(mem:read8(self.PC) + self.X, 0xff)
    self.PC = self.PC + 1
    return mem:read8(b1) + mem:read8(band(b1 + 1, 0xff)) * 0x100
  end
  local indy
  indy = function(self)
    local b1 = mem:read8(self.PC)
    self.PC = self.PC + 1
    return band((mem:read8(b1) + mem:read8(band(b1 + 1, 0xff)) * 0x100 + self.Y), 0xffff)
  end
  local indyp
  indyp = function(self)
    local b1 = mem:read8(self.PC)
    self.PC = self.PC + 1
    local base = mem:read8(b1) + mem:read8(band(b1 + 1, 0xff)) * 0x100
    local addr = band((base + self.Y), 0xffff)
    self._cycles = self._cycles + penalty(base, addr)
    return addr
  end
  local JAM
  JAM = function(self)
    return error(("CPU JAM @%0x4"):format(self.PC))
  end
  local CLC
  CLC = function(self)
    return cpu:setC(0x00)
  end
  local CLD
  CLD = function(self)
    return cpu:setD(0x00)
  end
  local CLV
  CLV = function(self)
    return cpu:setV(0x00)
  end
  local SEC
  SEC = function(self)
    return cpu:setC(0x01)
  end
  local SED
  SED = function(self)
    return cpu:setD(0x08)
  end
  local PHA
  PHA = function(self)
    return cpu:push8(self.A)
  end
  local PHP
  PHP = function(self)
    return cpu:push8(bor(self.P, 0x10))
  end
  local TAX
  TAX = function(self)
    self.X = cpu:updateZN(self.A)
  end
  local TAY
  TAY = function(self)
    self.Y = cpu:updateZN(self.A)
  end
  local TXA
  TXA = function(self)
    self.A = cpu:updateZN(self.X)
  end
  local TYA
  TYA = function(self)
    self.A = cpu:updateZN(self.Y)
  end
  local TSX
  TSX = function(self)
    self.X = cpu:updateZN(self.SP)
  end
  local TXS
  TXS = function(self)
    self.SP = self.X
  end
  local INX
  INX = function(self)
    self.X = cpu:updateZN(band(self.X + 1, 0xff))
  end
  local DEX
  DEX = function(self)
    self.X = cpu:updateZN(band(self.X - 1, 0xff))
  end
  local INY
  INY = function(self)
    self.Y = cpu:updateZN(band(self.Y + 1, 0xff))
  end
  local DEY
  DEY = function(self)
    self.Y = cpu:updateZN(band(self.Y - 1, 0xff))
  end
  local PLA
  PLA = function(self)
    self.A = cpu:updateZN(cpu:pop8())
  end
  local RTS
  RTS = function(self)
    self.PC = cpu:pop16() + 1
  end
  local ORA
  ORA = function(am)
    return function(self)
      self.A = cpu:updateZN(bor(self.A, read8(am(self))))
    end
  end
  local AND
  AND = function(am)
    return function(self)
      self.A = cpu:updateZN(band(self.A, read8(am(self))))
    end
  end
  local EOR
  EOR = function(am)
    return function(self)
      self.A = cpu:updateZN(bxor(self.A, read8(am(self))))
    end
  end
  local LDA
  LDA = function(am)
    return function(self)
      self.A = cpu:updateZN(read8(am(self)))
    end
  end
  local LDX
  LDX = function(am)
    return function(self)
      self.X = cpu:updateZN(read8(am(self)))
    end
  end
  local LDY
  LDY = function(am)
    return function(self)
      self.Y = cpu:updateZN(read8(am(self)))
    end
  end
  local SAX
  SAX = function(am)
    return function(self)
      return write8(am(self), band(self.A, self.X))
    end
  end
  local STA
  STA = function(am)
    return function(self)
      return write8(am(self), self.A)
    end
  end
  local STX
  STX = function(am)
    return function(self)
      return write8(am(self), self.X)
    end
  end
  local STY
  STY = function(am)
    return function(self)
      return write8(am(self), self.Y)
    end
  end
  local BRK
  BRK = function(self)
    cpu:push16(self.PC + 1)
    cpu:push8(bor(self.P, 0x30))
    cpu:setI(0x04)
    self.PC = read16(0xfffe)
  end
  local NOP
  NOP = function(am)
    return function(self)
      am(self)
      return nil
    end
  end
  local SEI
  SEI = function(self)
    return cpu:setI(0x04)
  end
  local PLP
  PLP = function(self)
    self.P = bor(band(cpu:pop8(), 0xef), 0x20)
  end
  local RTI
  RTI = function(self)
    self.P, self.PC = bor(band(cpu:pop8(), 0xef), 0x20), cpu:pop16()
    return cpu:_pollInterrupts()
  end
  local CLI
  CLI = function(self)
    cpu:setI(0x00)
    self._delay_irq = self._delay_irq + 1
  end
  local JSR
  JSR = function(self)
    local base = read16(self.PC)
    self.PC = self.PC + 2
    cpu:push16(band(self.PC - 1, 0xffff))
    self.PC = base
  end
  local ASL
  ASL = function(am, r, w)
    return function(self)
      local addr = am(self)
      local v = lshift(r(addr), 1)
      cpu:setC(rshift(v, 8))
      local a = cpu:updateZN(band(v, 0xff))
      return w(addr, a)
    end
  end
  local SLO
  SLO = function(am, r, w)
    return function(self)
      local addr = am(self)
      local b = lshift(r(addr), 1)
      w(addr, b)
      local v = bor(self.A, b)
      cpu:setC(rshift(b, 8))
      self.A = cpu:updateZN(band(v, 0xff))
    end
  end
  local BIT
  BIT = function(am)
    return function(self)
      local v = read8(am(self))
      cpu:updateZ(band(self.A, v))
      self.P = bor(band(self.P, 0x3f), band(v, 0xc0))
    end
  end
  local ROL
  ROL = function(am, r, w)
    return function(self)
      local addr = am(self)
      local a = r(addr)
      local v = bor(lshift(a, 1), band(self.P, 0x01))
      cpu:setC(rshift(v, 8))
      a = band(v, 0xff)
      w(addr, a)
      return cpu:updateZN(a)
    end
  end
  local RLA
  RLA = function(am, r, w)
    return function(self)
      local addr = am(self)
      local a = r(addr)
      local v = bor(a * 2, band(self.P, 0x01))
      w(addr, v)
      cpu:setC(rshift(v, 8))
      a = band(band(self.A, v), 0xff)
      self.A = cpu:updateZN(a)
    end
  end
  local LSR
  LSR = function(am, r, w)
    return function(self)
      local addr = am(self)
      local a = r(addr)
      local v = rshift(a, 1)
      cpu:setC(band(a, 0x01))
      w(addr, v)
      return cpu:updateZN(v)
    end
  end
  local SRE
  SRE = function(am, r, w)
    return function(self)
      local addr = am(self)
      local a = r(addr)
      local v = rshift(a, 1)
      w(addr, v)
      cpu:setC(band(a, 0x01))
      v = bxor(v, self.A)
      self.A = cpu:updateZN(v)
    end
  end
  local ROR
  ROR = function(am, r, w)
    return function(self)
      local addr = am(self)
      local a = r(addr)
      local v = rshift(a, 1) + band(self.P, 0x01) * 0x80
      cpu:setC(band(a, 0x01))
      a = band(v, 0xff)
      w(addr, a)
      return cpu:updateZN(a)
    end
  end
  local adc
  if BCD then
    local min, max
    do
      local _obj_0 = math
      min, max = _obj_0.min, _obj_0.max
    end
    adc = function(a, m, carry, bcd)
      local res = a + m + carry
      local v = rshift(band(bxor(res, m), band(bxor(res, a), 0x80)), 1)
      res = res + (bcd * 0x06 * max(0x00, min(0x01, (band(a, 0x0f) + band(m, 0x0f) + carry) - 0x09)))
      res = res + (bcd * 0x60 * max(0x00, min(0x01, res - 0x99)))
      local c = band(rshift(res, 8), 0x01)
      return band(res, 0xff), v, c
    end
  else
    adc = function(a, m, carry)
      local res = a + m + carry
      local v = rshift(band(bxor(res, m), band(bxor(res, a), 0x80)), 1)
      local c = band(rshift(res, 8), 0x01)
      return band(res, 0xff), v, c
    end
  end
  local ADC
  ADC = function(am)
    return function(self)
      local m = read8(am(self))
      local a, v, c = adc(self.A, m, band(self.P, 0x01), rshift(band(self.P, 0x08), 3))
      cpu:setV(v)
      cpu:setC(c)
      self.A = cpu:updateZN(a)
    end
  end
  local sbc
  if BCD then
    local min, max
    do
      local _obj_0 = math
      min, max = _obj_0.min, _obj_0.max
    end
    sbc = function(a, m, carry, bcd)
      local res = band(a - m - (1 - carry), 0xffff)
      local v = rshift(band(bxor(m, a), band(bxor(res, a), 0x80)), 1)
      res = res - (bcd * 0x06 * max(0x00, min(0x01, band(m, 0x0f) - band(a, 0x0f) + (1 - carry))))
      res = res - (bcd * 0x60 * max(0x00, min(0x01, res - 0x99)))
      local c = 1 - band(rshift(res, 8), 1)
      return band(res, 0xff), v, c
    end
  else
    sbc = function(a, m, carry)
      local res = band(a - m - (1 - carry), 0xffff)
      local v = rshift(band(bxor(m, a), band(bxor(res, a), 0x80)), 1)
      local c = 1 - band(rshift(res, 8), 1)
      return band(res, 0xff), v, c
    end
  end
  local SBC
  SBC = function(am)
    return function(self)
      local m = read8(am(self))
      local a, v, c = sbc(self.A, m, band(self.P, 0x01), rshift(band(self.P, 0x08), 3))
      cpu:setV(v)
      cpu:setC(c)
      self.A = cpu:updateZN(a)
    end
  end
  local CMP
  CMP = function(am)
    local min, max
    do
      local _obj_0 = math
      min, max = _obj_0.min, _obj_0.max
    end
    return function(self)
      local val = self.A - read8(am(self))
      cpu:setC(max(0x00, min(val + 1, 0x01)))
      return cpu:updateZN(val)
    end
  end
  local CPX
  CPX = function(am)
    local min, max
    do
      local _obj_0 = math
      min, max = _obj_0.min, _obj_0.max
    end
    return function(self)
      local val = self.X - read8(am(self))
      cpu:setC(max(0x00, min(val + 1, 0x01)))
      return cpu:updateZN(val)
    end
  end
  local CPY
  CPY = function(am)
    local min, max
    do
      local _obj_0 = math
      min, max = _obj_0.min, _obj_0.max
    end
    return function(self)
      local val = self.Y - read8(am(self))
      cpu:setC(max(0x00, min(val + 1, 0x01)))
      return cpu:updateZN(val)
    end
  end
  local DEC
  DEC = function(am)
    return function(self)
      local addr = am(self)
      local v = cpu:updateZN(band(read8(addr) - 1, 0xff))
      return write8(addr, v)
    end
  end
  local INC
  INC = function(am)
    return function(self)
      local addr = am(self)
      local v = cpu:updateZN(band(read8(addr) + 1, 0xff))
      return write8(addr, v)
    end
  end
  local trap = CPU_TEST and function(self)
    return error(("trapped: %04x @ %d"):format(self.PC - 1, tonumber(self._instCounter)))
  end
  local BRANCH
  BRANCH = function(flag, ifset)
    if CPU_TEST then
      return function(self)
        local base = rel(self)
        if not (cpu:testStatus(flag) == ifset) then
          return 
        end
        if base == -2 then
          trap(self)
        end
        local addr = band(self.PC + base, 0xffff)
        self._cycles = self._cycles + (1 + penalty(self.PC, addr))
        self.PC = addr
      end
    else
      return function(self)
        local base = rel(self)
        if not (cpu:testStatus(flag) == ifset) then
          return 
        end
        local addr = band(self.PC + base, 0xffff)
        self._cycles = self._cycles + (1 + penalty(self.PC, addr))
        self.PC = addr
      end
    end
  end
  local JMP
  JMP = function(am)
    if CPU_TEST then
      return function(self)
        local addr = am(self)
        if addr == self.PC - 3 then
          if addr == 0x3469 then
            CPU_TEST()
          else
            trap(self)
          end
        end
        self.PC = addr
      end
    else
      return function(self)
        self.PC = am(self)
      end
    end
  end
  local LAX
  LAX = function(am)
    return function(self)
      self.X = cpu:updateZN(read8(am(self)))
      self.A = self.X
    end
  end
  local DCP
  DCP = function(am)
    local min, max
    do
      local _obj_0 = math
      min, max = _obj_0.min, _obj_0.max
    end
    return function(self)
      local addr = am(self)
      local v = band(read8(addr) - 1, 0xff)
      write8(addr, v)
      local cmp = self.A - v
      cpu:setC(max(0x00, min(cmp + 1, 0x01)))
      return cpu:updateZN(cmp)
    end
  end
  local ISB
  ISB = function(am)
    return function(self)
      local addr = am(self)
      local m = band(read8(addr) + 1, 0xff)
      write8(addr, m)
      local a, v, c = sbc(self.A, m, band(self.P, 0x01), rshift(band(self.P, 0x08), 3))
      cpu:setV(v)
      cpu:setC(c)
      self.A = cpu:updateZN(a)
    end
  end
  local RRA
  RRA = function(am)
    return function(self)
      local addr = am(self)
      local m = read8(addr)
      local r = rshift(m, 1) + band(self.P, 0x01) * 0x80
      local c = band(m, 0x01)
      local a = band(r, 0xff)
      write8(addr, a)
      local v
      a, v, c = adc(self.A, a, c, rshift(band(self.P, 0x08), 3))
      cpu:setV(v)
      cpu:setC(c)
      self.A = cpu:updateZN(a)
    end
  end
  local opcodes = {
    [0x00] = {
      1,
      7,
      false,
      BRK
    },
    [0x01] = {
      2,
      6,
      false,
      ORA(indx)
    },
    [0x02] = {
      1,
      2,
      true,
      JAM
    },
    [0x03] = {
      2,
      8,
      true,
      SLO(indx, read8, write8)
    },
    [0x04] = {
      2,
      3,
      true,
      NOP(zp)
    },
    [0x05] = {
      2,
      3,
      false,
      ORA(zp)
    },
    [0x06] = {
      2,
      5,
      false,
      ASL(zp, read8, write8)
    },
    [0x07] = {
      2,
      5,
      true,
      SLO(zp, read8, write8)
    },
    [0x08] = {
      1,
      3,
      false,
      PHP
    },
    [0x09] = {
      2,
      2,
      false,
      ORA(imm)
    },
    [0x0a] = {
      1,
      2,
      false,
      ASL(accum, accum, writea)
    },
    [0x0c] = {
      3,
      4,
      true,
      NOP(ab)
    },
    [0x0d] = {
      3,
      4,
      false,
      ORA(ab)
    },
    [0x0e] = {
      3,
      6,
      false,
      ASL(ab, read8, write8)
    },
    [0x0f] = {
      3,
      6,
      true,
      SLO(ab, read8, write8)
    },
    [0x10] = {
      2,
      2,
      false,
      BRANCH(0x80, false)
    },
    [0x11] = {
      2,
      5,
      false,
      ORA(indyp)
    },
    [0x12] = {
      1,
      2,
      true,
      JAM
    },
    [0x13] = {
      2,
      8,
      true,
      SLO(indy, read8, write8)
    },
    [0x14] = {
      2,
      4,
      true,
      NOP(zpx)
    },
    [0x15] = {
      2,
      4,
      false,
      ORA(zpx)
    },
    [0x16] = {
      2,
      6,
      false,
      ASL(zpx, read8, write8)
    },
    [0x17] = {
      2,
      6,
      true,
      SLO(zpx, read8, write8)
    },
    [0x18] = {
      1,
      2,
      false,
      CLC
    },
    [0x19] = {
      3,
      4,
      false,
      ORA(abyp)
    },
    [0x1a] = {
      1,
      2,
      true,
      NOP(imp)
    },
    [0x1b] = {
      3,
      7,
      true,
      SLO(aby, read8, write8)
    },
    [0x1c] = {
      3,
      4,
      true,
      NOP(abxp)
    },
    [0x1d] = {
      3,
      4,
      false,
      ORA(abxp)
    },
    [0x1e] = {
      3,
      7,
      false,
      ASL(abx, read8, write8)
    },
    [0x1f] = {
      3,
      7,
      true,
      SLO(abx, read8, write8)
    },
    [0x20] = {
      3,
      6,
      false,
      JSR
    },
    [0x21] = {
      2,
      6,
      false,
      AND(indx)
    },
    [0x22] = {
      1,
      2,
      true,
      JAM
    },
    [0x23] = {
      2,
      8,
      true,
      RLA(indx, read8, write8)
    },
    [0x24] = {
      2,
      3,
      false,
      BIT(zp)
    },
    [0x25] = {
      2,
      3,
      false,
      AND(zp)
    },
    [0x26] = {
      2,
      5,
      false,
      ROL(zp, read8, write8)
    },
    [0x27] = {
      2,
      5,
      true,
      RLA(zp, read8, write8)
    },
    [0x28] = {
      1,
      4,
      false,
      PLP
    },
    [0x29] = {
      2,
      2,
      false,
      AND(imm)
    },
    [0x2a] = {
      1,
      2,
      false,
      ROL(accum, accum, writea)
    },
    [0x2c] = {
      3,
      4,
      false,
      BIT(ab)
    },
    [0x2d] = {
      3,
      4,
      false,
      AND(ab)
    },
    [0x2e] = {
      3,
      6,
      false,
      ROL(ab, read8, write8)
    },
    [0x2f] = {
      3,
      6,
      true,
      RLA(ab, read8, write8)
    },
    [0x30] = {
      2,
      2,
      false,
      BRANCH(0x80, true)
    },
    [0x31] = {
      2,
      5,
      false,
      AND(indyp)
    },
    [0x32] = {
      1,
      2,
      true,
      JAM
    },
    [0x33] = {
      2,
      8,
      true,
      RLA(indy, read8, write8)
    },
    [0x34] = {
      2,
      4,
      true,
      NOP(zpx)
    },
    [0x35] = {
      2,
      4,
      false,
      AND(zpx)
    },
    [0x36] = {
      2,
      6,
      false,
      ROL(zpx, read8, write8)
    },
    [0x37] = {
      2,
      6,
      true,
      RLA(zpx, read8, write8)
    },
    [0x38] = {
      1,
      2,
      false,
      SEC
    },
    [0x39] = {
      3,
      4,
      false,
      AND(abyp)
    },
    [0x3a] = {
      1,
      2,
      true,
      NOP(imp)
    },
    [0x3b] = {
      3,
      7,
      true,
      RLA(aby, read8, write8)
    },
    [0x3c] = {
      3,
      4,
      true,
      NOP(abxp)
    },
    [0x3d] = {
      3,
      4,
      false,
      AND(abxp)
    },
    [0x3e] = {
      3,
      7,
      false,
      ROL(abx, read8, write8)
    },
    [0x3f] = {
      3,
      7,
      true,
      RLA(abx, read8, write8)
    },
    [0x40] = {
      1,
      6,
      false,
      RTI
    },
    [0x41] = {
      2,
      6,
      false,
      EOR(indx)
    },
    [0x42] = {
      1,
      2,
      true,
      JAM
    },
    [0x43] = {
      2,
      8,
      true,
      SRE(indx, read8, write8)
    },
    [0x44] = {
      2,
      3,
      true,
      NOP(zp)
    },
    [0x45] = {
      2,
      3,
      false,
      EOR(zp)
    },
    [0x46] = {
      2,
      5,
      false,
      LSR(zp, read8, write8)
    },
    [0x47] = {
      2,
      5,
      true,
      SRE(zp, read8, write8)
    },
    [0x48] = {
      1,
      3,
      false,
      PHA
    },
    [0x49] = {
      2,
      2,
      false,
      EOR(imm)
    },
    [0x4a] = {
      1,
      2,
      false,
      LSR(accum, accum, writea)
    },
    [0x4c] = {
      3,
      3,
      false,
      JMP(ab)
    },
    [0x4d] = {
      3,
      4,
      false,
      EOR(ab)
    },
    [0x4e] = {
      3,
      6,
      false,
      LSR(ab, read8, write8)
    },
    [0x4f] = {
      3,
      6,
      true,
      SRE(ab, read8, write8)
    },
    [0x50] = {
      2,
      2,
      false,
      BRANCH(0x40, false)
    },
    [0x51] = {
      2,
      5,
      false,
      EOR(indyp)
    },
    [0x52] = {
      1,
      2,
      true,
      JAM
    },
    [0x53] = {
      2,
      8,
      true,
      SRE(indy, read8, write8)
    },
    [0x54] = {
      2,
      4,
      true,
      NOP(zpx)
    },
    [0x55] = {
      2,
      4,
      false,
      EOR(zpx)
    },
    [0x56] = {
      2,
      6,
      false,
      LSR(zpx, read8, write8)
    },
    [0x57] = {
      2,
      6,
      true,
      SRE(zpx, read8, write8)
    },
    [0x58] = {
      1,
      2,
      false,
      CLI
    },
    [0x59] = {
      3,
      4,
      false,
      EOR(abyp)
    },
    [0x5a] = {
      1,
      2,
      true,
      NOP(imp)
    },
    [0x5b] = {
      3,
      7,
      true,
      SRE(aby, read8, write8)
    },
    [0x5c] = {
      3,
      4,
      true,
      NOP(abxp)
    },
    [0x5d] = {
      3,
      4,
      false,
      EOR(abxp)
    },
    [0x5e] = {
      3,
      7,
      false,
      LSR(abx, read8, write8)
    },
    [0x5f] = {
      3,
      7,
      true,
      SRE(abx, read8, write8)
    },
    [0x60] = {
      1,
      6,
      false,
      RTS
    },
    [0x61] = {
      2,
      6,
      false,
      ADC(indx)
    },
    [0x62] = {
      1,
      2,
      true,
      JAM
    },
    [0x63] = {
      2,
      8,
      true,
      RRA(indx)
    },
    [0x64] = {
      2,
      3,
      true,
      NOP(zp)
    },
    [0x65] = {
      2,
      3,
      false,
      ADC(zp)
    },
    [0x66] = {
      2,
      5,
      false,
      ROR(zp, read8, write8)
    },
    [0x67] = {
      2,
      5,
      true,
      RRA(zp)
    },
    [0x68] = {
      1,
      4,
      false,
      PLA
    },
    [0x69] = {
      2,
      2,
      false,
      ADC(imm)
    },
    [0x6a] = {
      1,
      2,
      false,
      ROR(accum, accum, writea)
    },
    [0x6c] = {
      3,
      5,
      false,
      JMP(ind)
    },
    [0x6d] = {
      3,
      4,
      false,
      ADC(ab)
    },
    [0x6e] = {
      3,
      6,
      false,
      ROR(ab, read8, write8)
    },
    [0x6f] = {
      3,
      6,
      true,
      RRA(ab)
    },
    [0x70] = {
      2,
      2,
      false,
      BRANCH(0x40, true)
    },
    [0x71] = {
      2,
      5,
      false,
      ADC(indyp)
    },
    [0x72] = {
      1,
      2,
      true,
      JAM
    },
    [0x73] = {
      2,
      8,
      true,
      RRA(indy)
    },
    [0x74] = {
      2,
      4,
      true,
      NOP(zpx)
    },
    [0x75] = {
      2,
      4,
      false,
      ADC(zpx)
    },
    [0x76] = {
      2,
      6,
      false,
      ROR(zpx, read8, write8)
    },
    [0x77] = {
      2,
      6,
      true,
      RRA(zpx)
    },
    [0x78] = {
      1,
      2,
      false,
      SEI
    },
    [0x79] = {
      3,
      4,
      false,
      ADC(abyp)
    },
    [0x7a] = {
      1,
      2,
      true,
      NOP(imp)
    },
    [0x7b] = {
      3,
      7,
      true,
      RRA(aby)
    },
    [0x7c] = {
      3,
      4,
      true,
      NOP(abxp)
    },
    [0x7d] = {
      3,
      4,
      false,
      ADC(abxp)
    },
    [0x7e] = {
      3,
      7,
      false,
      ROR(abx, read8, write8)
    },
    [0x7f] = {
      3,
      7,
      true,
      RRA(abx)
    },
    [0x80] = {
      2,
      2,
      true,
      NOP(imm)
    },
    [0x81] = {
      2,
      6,
      false,
      STA(indx)
    },
    [0x82] = {
      2,
      2,
      true,
      NOP(imm)
    },
    [0x83] = {
      2,
      6,
      true,
      SAX(indx)
    },
    [0x84] = {
      2,
      3,
      false,
      STY(zp)
    },
    [0x85] = {
      2,
      3,
      false,
      STA(zp)
    },
    [0x86] = {
      2,
      3,
      false,
      STX(zp)
    },
    [0x87] = {
      2,
      3,
      true,
      SAX(zp)
    },
    [0x88] = {
      1,
      2,
      false,
      DEY
    },
    [0x89] = {
      2,
      2,
      true,
      NOP(imm)
    },
    [0x8a] = {
      1,
      2,
      false,
      TXA
    },
    [0x8c] = {
      3,
      4,
      false,
      STY(ab)
    },
    [0x8d] = {
      3,
      4,
      false,
      STA(ab)
    },
    [0x8e] = {
      3,
      4,
      false,
      STX(ab)
    },
    [0x8f] = {
      3,
      4,
      true,
      SAX(ab)
    },
    [0x90] = {
      2,
      2,
      false,
      BRANCH(0x01, false)
    },
    [0x91] = {
      2,
      6,
      false,
      STA(indy)
    },
    [0x92] = {
      1,
      2,
      true,
      JAM
    },
    [0x94] = {
      2,
      4,
      false,
      STY(zpx)
    },
    [0x95] = {
      2,
      4,
      false,
      STA(zpx)
    },
    [0x96] = {
      2,
      4,
      false,
      STX(zpy)
    },
    [0x97] = {
      2,
      4,
      true,
      SAX(zpy)
    },
    [0x98] = {
      1,
      2,
      false,
      TYA
    },
    [0x99] = {
      3,
      5,
      false,
      STA(aby)
    },
    [0x9a] = {
      1,
      2,
      false,
      TXS
    },
    [0x9d] = {
      3,
      5,
      false,
      STA(abx)
    },
    [0xa0] = {
      2,
      2,
      false,
      LDY(imm)
    },
    [0xa1] = {
      2,
      6,
      false,
      LDA(indx)
    },
    [0xa2] = {
      2,
      2,
      false,
      LDX(imm)
    },
    [0xa3] = {
      2,
      6,
      true,
      LAX(indx)
    },
    [0xa4] = {
      2,
      3,
      false,
      LDY(zp)
    },
    [0xa5] = {
      2,
      3,
      false,
      LDA(zp)
    },
    [0xa6] = {
      2,
      3,
      false,
      LDX(zp)
    },
    [0xa7] = {
      2,
      3,
      true,
      LAX(zp)
    },
    [0xa8] = {
      1,
      2,
      false,
      TAY
    },
    [0xa9] = {
      2,
      2,
      false,
      LDA(imm)
    },
    [0xaa] = {
      1,
      2,
      false,
      TAX
    },
    [0xab] = {
      2,
      2,
      true,
      LAX(imm)
    },
    [0xac] = {
      3,
      4,
      false,
      LDY(ab)
    },
    [0xad] = {
      3,
      4,
      false,
      LDA(ab)
    },
    [0xae] = {
      3,
      4,
      false,
      LDX(ab)
    },
    [0xaf] = {
      3,
      4,
      true,
      LAX(ab)
    },
    [0xb0] = {
      2,
      2,
      false,
      BRANCH(0x01, true)
    },
    [0xb1] = {
      2,
      5,
      false,
      LDA(indyp)
    },
    [0xb2] = {
      1,
      2,
      true,
      JAM
    },
    [0xb3] = {
      2,
      5,
      true,
      LAX(indyp)
    },
    [0xb4] = {
      2,
      4,
      false,
      LDY(zpx)
    },
    [0xb5] = {
      2,
      4,
      false,
      LDA(zpx)
    },
    [0xb6] = {
      2,
      4,
      false,
      LDX(zpy)
    },
    [0xb7] = {
      2,
      4,
      true,
      LAX(zpy)
    },
    [0xb8] = {
      1,
      2,
      false,
      CLV
    },
    [0xb9] = {
      3,
      4,
      false,
      LDA(abyp)
    },
    [0xba] = {
      1,
      2,
      false,
      TSX
    },
    [0xbc] = {
      3,
      4,
      false,
      LDY(abxp)
    },
    [0xbd] = {
      3,
      4,
      false,
      LDA(abxp)
    },
    [0xbe] = {
      3,
      4,
      false,
      LDX(abyp)
    },
    [0xbf] = {
      3,
      4,
      true,
      LAX(abyp)
    },
    [0xc0] = {
      2,
      2,
      false,
      CPY(imm)
    },
    [0xc1] = {
      2,
      6,
      false,
      CMP(indx)
    },
    [0xc2] = {
      2,
      2,
      true,
      NOP(imm)
    },
    [0xc3] = {
      2,
      8,
      true,
      DCP(indx)
    },
    [0xc4] = {
      2,
      3,
      false,
      CPY(zp)
    },
    [0xc5] = {
      2,
      3,
      false,
      CMP(zp)
    },
    [0xc6] = {
      2,
      5,
      false,
      DEC(zp)
    },
    [0xc7] = {
      2,
      5,
      true,
      DCP(zp)
    },
    [0xc8] = {
      1,
      2,
      false,
      INY
    },
    [0xc9] = {
      2,
      2,
      false,
      CMP(imm)
    },
    [0xca] = {
      1,
      2,
      false,
      DEX
    },
    [0xcc] = {
      3,
      4,
      false,
      CPY(ab)
    },
    [0xcd] = {
      3,
      4,
      false,
      CMP(ab)
    },
    [0xce] = {
      3,
      6,
      false,
      DEC(ab)
    },
    [0xcf] = {
      3,
      6,
      true,
      DCP(ab)
    },
    [0xd0] = {
      2,
      2,
      false,
      BRANCH(0x02, false)
    },
    [0xd1] = {
      2,
      5,
      false,
      CMP(indyp)
    },
    [0xd2] = {
      1,
      2,
      true,
      JAM
    },
    [0xd3] = {
      2,
      8,
      true,
      DCP(indy)
    },
    [0xd4] = {
      2,
      4,
      true,
      NOP(zpx)
    },
    [0xd5] = {
      2,
      4,
      false,
      CMP(zpx)
    },
    [0xd6] = {
      2,
      6,
      false,
      DEC(zpx)
    },
    [0xd7] = {
      2,
      6,
      true,
      DCP(zpx)
    },
    [0xd8] = {
      1,
      2,
      false,
      CLD
    },
    [0xd9] = {
      3,
      4,
      false,
      CMP(abyp)
    },
    [0xda] = {
      1,
      2,
      true,
      NOP(imp)
    },
    [0xdb] = {
      3,
      7,
      true,
      DCP(aby)
    },
    [0xdc] = {
      3,
      4,
      true,
      NOP(abxp)
    },
    [0xdd] = {
      3,
      4,
      false,
      CMP(abxp)
    },
    [0xde] = {
      3,
      7,
      false,
      DEC(abx)
    },
    [0xdf] = {
      3,
      7,
      true,
      DCP(abx)
    },
    [0xe0] = {
      2,
      2,
      false,
      CPX(imm)
    },
    [0xe1] = {
      2,
      6,
      false,
      SBC(indx)
    },
    [0xe2] = {
      2,
      2,
      true,
      NOP(imm)
    },
    [0xe3] = {
      2,
      8,
      true,
      ISB(indx)
    },
    [0xe4] = {
      2,
      3,
      false,
      CPX(zp)
    },
    [0xe5] = {
      2,
      3,
      false,
      SBC(zp)
    },
    [0xe6] = {
      2,
      5,
      false,
      INC(zp)
    },
    [0xe7] = {
      2,
      5,
      true,
      ISB(zp)
    },
    [0xe8] = {
      1,
      2,
      false,
      INX
    },
    [0xe9] = {
      2,
      2,
      false,
      SBC(imm)
    },
    [0xea] = {
      1,
      2,
      false,
      NOP(imp)
    },
    [0xeb] = {
      2,
      2,
      true,
      SBC(imm)
    },
    [0xec] = {
      3,
      4,
      false,
      CPX(ab)
    },
    [0xed] = {
      3,
      4,
      false,
      SBC(ab)
    },
    [0xee] = {
      3,
      6,
      false,
      INC(ab)
    },
    [0xef] = {
      3,
      6,
      true,
      ISB(ab)
    },
    [0xf0] = {
      2,
      2,
      false,
      BRANCH(0x02, true)
    },
    [0xf1] = {
      2,
      5,
      false,
      SBC(indyp)
    },
    [0xf2] = {
      1,
      2,
      true,
      JAM
    },
    [0xf3] = {
      2,
      8,
      true,
      ISB(indy)
    },
    [0xf4] = {
      2,
      4,
      true,
      NOP(zpx)
    },
    [0xf5] = {
      2,
      4,
      false,
      SBC(zpx)
    },
    [0xf6] = {
      2,
      6,
      false,
      INC(zpx)
    },
    [0xf7] = {
      2,
      6,
      true,
      ISB(zpx)
    },
    [0xf8] = {
      1,
      2,
      false,
      SED
    },
    [0xf9] = {
      3,
      4,
      false,
      SBC(abyp)
    },
    [0xfa] = {
      1,
      2,
      true,
      NOP(imp)
    },
    [0xfb] = {
      3,
      7,
      true,
      ISB(aby)
    },
    [0xfc] = {
      3,
      4,
      true,
      NOP(abxp)
    },
    [0xfd] = {
      3,
      4,
      false,
      SBC(abxp)
    },
    [0xfe] = {
      3,
      7,
      false,
      INC(abx)
    },
    [0xff] = {
      3,
      7,
      true,
      ISB(abx)
    }
  }
  local miss = 0
  for i = 0, 0xff do
    if not (opcodes[i]) then
      miss = miss + 1
      opcodes[i] = opcodes[0x02]
    end
  end
  return opcodes
end
