local ffi = require('ffi')
local band
band = require('bit').band
return function(nes)
  local cart = nes:getCart()
  local vram = nes:getPPU():getMem():getVRAM()
  local nametables = require('Cart.util').NTSelector(cart)
  local selectNT = nametables.selectNT
  do
    local _with_0 = cart
    local rom = _with_0:getPRGROM()
    local chr = _with_0:getCHRROM() or _with_0:getCHRRAM()
    local ROMMASK = _with_0:getPRGROMSize() - 1
    local read
    read = function(self, addr)
      return rom[band(addr, ROMMASK)]
    end
    local write
    write = function(self, addr, value) end
    local readVRAM
    readVRAM = function(self, addr)
      return vram[selectNT(addr) + band(addr, 0x3ff)]
    end
    local writeVRAM
    writeVRAM = function(self, addr, value)
      vram[selectNT(addr) + band(addr, 0x3ff)] = value
    end
    local readCHR
    do
      local CHRMASK = (_with_0:getCHRRAM() and _with_0:getCHRRAMSize() or _with_0:getCHRROMSize()) - 1
      readCHR = function(self, addr)
        return chr[band(addr, CHRMASK)]
      end
    end
    local writeCHR
    do
      if _with_0:getCHRRAM() then
        local CHRMASK = _with_0:getCHRRAMSize() - 1
        writeCHR = function(self, addr, val)
          chr[band(addr, CHRMASK)] = val
        end
      else
        writeCHR = function(self, addr, val) end
      end
    end
    return {
      read = read,
      write = write,
      readCHR = readCHR,
      writeCHR = writeCHR,
      readVRAM = readVRAM,
      writeVRAM = writeVRAM,
      nametables = nametables
    }
  end
end
