local keyboard
keyboard = love.keyboard
local rshift, lshift, band, bor
do
  local _obj_0 = require('bit')
  rshift, lshift, band, bor = _obj_0.rshift, _obj_0.lshift, _obj_0.band, _obj_0.bor
end
local A = 0
local B = 1
local SELECT = 2
local START = 3
local UP = 4
local DOWN = 5
local LEFT = 6
local RIGHT = 7
local IMPOSSIBLE = {
  [UP] = DOWN,
  [DOWN] = UP,
  [LEFT] = RIGHT,
  [RIGHT] = LEFT
}
local GAMEPADMAP = {
  a = A,
  x = B,
  back = SELECT,
  start = START,
  dpup = UP,
  dpdown = DOWN,
  dpleft = LEFT,
  dpright = RIGHT
}
local GAMEPADNOISE = .01
local GAMEPADDEADZONE = .1
local GAMEPADTHRESHOLD = .5
local prevAxes = { }
local Controller
do
  local _class_0
  local _base_0 = {
    keypressed = function(self, key)
      local btn = self._buttonMap[key]
      if not (btn) then
        return 
      end
      self._buttonState[btn] = true
      return true
    end,
    keyreleased = function(self, key)
      local btn = self._buttonMap[key]
      if not (btn) then
        return 
      end
      self._buttonState[btn] = false
      return true
    end,
    gamepadpressed = function(self, joystick, button)
      local btn = GAMEPADMAP[button]
      if not (btn) then
        return 
      end
      self._buttonState[btn] = true
      return true
    end,
    gamepadreleased = function(self, joystick, button)
      local btn = GAMEPADMAP[button]
      if not (btn) then
        return 
      end
      self._buttonState[btn] = false
      return true
    end,
    gamepadaxis = function(self, joystick, axis, value)
      if not (axis == 'leftx' or axis == 'lefty') then
        return 
      end
      local axisval = prevAxes[joystick]
      if not (axisval) then
        axisval = {
          leftx = axis == leftx and value or 0,
          lefty = axis == lefty and value or 0
        }
        prevAxes[joystick] = axisval
      end
      if math.abs(value) < GAMEPADNOISE then
        return 
      end
      local _exp_0 = axis
      if 'leftx' == _exp_0 then
        self._buttonState[LEFT] = value <= -GAMEPADTHRESHOLD
        self._buttonState[RIGHT] = value > GAMEPADTHRESHOLD
        axisval.leftx = value
        return true
      elseif 'lefty' == _exp_0 then
        self._buttonState[UP] = value <= -GAMEPADTHRESHOLD
        self._buttonState[DOWN] = value > GAMEPADTHRESHOLD
        axisval.lefty = value
        return true
      end
    end,
    step = function(self) end,
    read = function(self)
      local b = bor(band(self._data, 0xb0), 0x40)
      if self._strobe then
        return bor(self._buttonState[A] and 1 or 0, b)
      end
      local val = band(self._state, 0x01)
      self._state = rshift(bor(self._state, 0xff00), 1)
      return bor(val, b)
    end,
    write = function(self, val)
      self._data = val
      self._strobe = band(val, 0x01) == 0x01
      if self._strobe then
        return 
      end
      self._state = 0
      for i = A, RIGHT do
        local _continue_0 = false
        repeat
          if not self._buttonState[i] or self._buttonState[IMPOSSIBLE[i]] then
            _continue_0 = true
            break
          end
          self._state = bor(self._state, lshift(1, i))
          _continue_0 = true
        until true
        if not _continue_0 then
          break
        end
      end
    end
  }
  _base_0.__index = _base_0
  _class_0 = setmetatable({
    __init = function(self, _apu, _addr, keys)
      if keys == nil then
        keys = { }
      end
      self._apu, self._addr = _apu, _addr
      self._buttonMap = {
        [A] = keys.a,
        [B] = keys.b,
        [SELECT] = keys.select,
        [START] = keys.start,
        [UP] = keys.up,
        [DOWN] = keys.down,
        [LEFT] = keys.left,
        [RIGHT] = keys.right
      }
      for k, v in pairs(self._buttonMap) do
        self._buttonMap[v] = k
      end
      do
        local _tbl_0 = { }
        for i = A, RIGHT do
          _tbl_0[i] = false
        end
        self._buttonState = _tbl_0
      end
      self._state = 0
      self._data = 0
      self._strobe = false
    end,
    __base = _base_0,
    __name = "Controller"
  }, {
    __index = _base_0,
    __call = function(cls, ...)
      local _self_0 = setmetatable({}, _base_0)
      cls.__init(_self_0, ...)
      return _self_0
    end
  })
  _base_0.__class = _class_0
  Controller = _class_0
  return _class_0
end
