local filesystem
filesystem = love.filesystem
local band
band = require('bit').band
local MNEMONICS = require('CPU.mnemonics')
local OPCODES = require('CPU.opcodes')
local logfile = nil
log = function(fmt, ...)
  logfile = logfile or filesystem.newFile('lnes.log', 'w')
  do
    logfile:write(fmt:format(...))
    logfile:write('\n')
    return logfile
  end
end
local formatStatus
formatStatus = function(s)
  return ("%s%s%s%s%s%s%s%s"):format((band(s, 0x80) > 0 and 'N' or '.'), (band(s, 0x40) > 0 and 'V' or '.'), '-', (band(s, 0x10) > 0 and 'B' or '.'), (band(s, 0x08) > 0 and 'D' or '.'), (band(s, 0x04) > 0 and 'I' or '.'), (band(s, 0x02) > 0 and 'Z' or '.'), (band(s, 0x01) > 0 and 'C' or '.'))
end
local formatInst
formatInst = function(mem, pc, size)
  local ops
  do
    local _accum_0 = { }
    local _len_0 = 1
    for i = 0, size do
      _accum_0[_len_0] = ("%02X"):format(mem:read8(pc + i))
      _len_0 = _len_0 + 1
    end
    ops = _accum_0
  end
  return table.concat(ops, ' ')
end
local logLine
logLine = function(state, op, size, mem, ppux, ppuy)
  do
    local _with_0 = state
    local inst = formatInst(mem, _with_0.PC, size)
    log(("%04X  %-9s %-31s A:%02X X:%02X Y:%02X P:%02X SP:%02X PPU:%3d,%3d CYC:%-8d %s"):format(_with_0.PC, inst, MNEMONICS[op], _with_0.A, _with_0.X, _with_0.Y, _with_0.P, _with_0.SP, ppux, ppuy, tonumber(_with_0._cycleCounter), formatStatus(_with_0.P)))
    return _with_0
  end
end
FETCHHOOK = function(state, op, mem, opcodes)
  local px, py = nes:getPPU():getState()
  px = px - 3
  if px < 0 then
    py = py - 1
    px = px + 341
    if py < 0 then
      py = 261
    end
  end
  local size = opcodes[op][1] - 1
  return logLine(state, op, size, mem, px, py)
end
