local ffi = require('ffi')
local band, rshift
do
  local _obj_0 = require('bit')
  band, rshift = _obj_0.band, _obj_0.rshift
end
local min, max
do
  local _obj_0 = math
  min, max = _obj_0.min, _obj_0.max
end
local selectIO
selectIO = function(addr)
  return min(2, rshift(addr, 13)) + min(1, max(0, addr - 0x401f))
end
return function(mapper, ppu, apu)
  local ram = ffi.new('uint8_t[?]', 0x0800)
  local read = {
    [0] = function(addr)
      return ram[band(addr, 0x07ff)]
    end,
    [1] = function(addr)
      return ppu:read(band(addr, 0x0007))
    end,
    [2] = function(addr)
      return apu:read(band(addr, 0x001f))
    end,
    [3] = function(addr)
      return mapper:read(addr)
    end
  }
  local write = {
    [0] = function(addr, value)
      ram[band(addr, 0x07ff)] = value
    end,
    [1] = function(addr, value)
      return ppu:write(band(addr, 0x0007), value)
    end,
    [2] = function(addr, value)
      return apu:write(band(addr, 0x001f), value)
    end,
    [3] = function(addr, value)
      return mapper:write(addr, value)
    end
  }
  return {
    read8 = function(self, addr)
      return read[selectIO(addr)](addr)
    end,
    write8 = function(self, addr, value)
      return write[selectIO(addr)](addr, value)
    end,
    read16 = function(self, addr)
      return self:read8(addr) + self:read8(addr + 1) * 0x100
    end,
    write16 = function(self, addr, value)
      self:write8(addr, band(rshift(value, 8), 0xff))
      return self:write8(addr + 1, band(value, 0xff))
    end
  }
end
