local ffi = require('ffi')
local band, rshift, bxor
do
  local _obj_0 = require('bit')
  band, rshift, bxor = _obj_0.band, _obj_0.rshift, _obj_0.bxor
end
local min, max
do
  local _obj_0 = math
  min, max = _obj_0.min, _obj_0.max
end
local PALMAP
do
  local p = ffi.new('int8_t[32]')
  for i = 0, 31 do
    p[i] = i
  end
  p[0x10] = p[0x00]
  p[0x14] = p[0x04]
  p[0x18] = p[0x08]
  p[0x1c] = p[0x0c]
  PALMAP = p
end
return function()
  local vram = ffi.new('uint8_t[?]', 0x0800)
  local palette = ffi.new('uint8_t[?]', 0x0020)
  local mapper = nil
  local setMapper
  setMapper = function(self, m)
    mapper = m
  end
  local selectIO
  selectIO = function(addr)
    return rshift(addr, 13) + max(0, min(1, addr - 0x3eff))
  end
  local readCHR
  readCHR = function(addr)
    return mapper:readCHR(addr)
  end
  local readVRAM
  readVRAM = function(addr)
    return mapper:readVRAM(addr)
  end
  local readPAL
  readPAL = function(addr)
    return palette[PALMAP[band(addr, 0x1f)]]
  end
  local writeCHR
  writeCHR = function(addr, value)
    return mapper:writeCHR(addr, value)
  end
  local writeVRAM
  writeVRAM = function(addr, value)
    return mapper:writeVRAM(addr, value)
  end
  local writePAL
  writePAL = function(addr, value)
    palette[PALMAP[band(addr, 0x1f)]] = value
  end
  local read = setmetatable({
    [0] = readCHR,
    [1] = readVRAM,
    [2] = readPAL
  }, {
    __index = function(self, k)
      return function(addr)
        return error(("BUG: read from %04x: no reader at %d"):format(addr, k))
      end
    end
  })
  local write = setmetatable({
    [0] = writeCHR,
    [1] = writeVRAM,
    [2] = writePAL
  }, {
    __index = function(self, k)
      return function(addr)
        return error(("BUG: write to %04x: no writer at %d"):format(addr, k))
      end
    end
  })
  return {
    setMapper = setMapper,
    readCHR = readCHR,
    readVRAM = readVRAM,
    readPAL = readPAL,
    writeCHR = writeCHR,
    writeVRAM = writeVRAM,
    writePAL = writePAL,
    getVRAM = function()
      return vram
    end,
    read = function(addr)
      return read[selectIO(addr)](addr)
    end,
    write = function(addr, value)
      return write[selectIO(addr)](addr, value)
    end
  }
end
