local ffi = require('ffi')
local floor
floor = math.floor
local timer
timer = love.timer
printf = function(fmt, ...)
  return print(fmt:format(...))
end
local Cart = require('Cart')
local Bus = require('Mem.Bus')
local CPU = require('CPU')
local PPU = require('PPU')
local APU = require('APU')
local Controller = require('Controller')
local NES
do
  local _class_0
  local _base_0 = {
    PAL = 1662607,
    NTSC = 1789773,
    reset = function(self, addr)
      self._ppu:reset()
      self._cpu:reset(addr, 0x24)
      self._phase = 0
    end,
    isPAL = function(self)
      return self._isPAL
    end,
    getMapper = function(self)
      return self._mapper
    end,
    getBus = function(self)
      return self._bus
    end,
    getCPU = function(self)
      return self._cpu
    end,
    getPPU = function(self)
      return self._ppu
    end,
    getAPU = function(self)
      return self._apu
    end,
    getController1 = function(self)
      return self._ctrl1
    end,
    getController2 = function(self)
      return self._ctrl2
    end,
    getCart = function(self)
      return self._cart
    end,
    getScreen = function(self)
      return self._screen
    end,
    setFrameCallback = function(self, _frameCallback)
      self._frameCallback = _frameCallback
    end,
    nextFrame = function(self)
      return self:_frameCallback()
    end,
    keypressed = function(self, key)
      return self._ctrl1:keypressed(key) or self._ctrl2:keypressed(key)
    end,
    keyreleased = function(self, key)
      return self._ctrl1:keyreleased(key) or self._ctrl2:keyreleased(key)
    end,
    gamepadpressed = function(self, joystick, button)
      return self._ctrl1:gamepadpressed(joystick, button) or self._ctrl2:gamepadpressed(joystick, button)
    end,
    gamepadreleased = function(self, joystick, button)
      return self._ctrl1:gamepadreleased(joystick, button) or self._ctrl2:gamepadreleased(joystick, button)
    end,
    gamepadaxis = function(self, joystick, axis, value)
      return self._ctrl1:gamepadaxis(joystick, axis, value) or self._ctrl2:gamepadaxis(joystick, axis, value)
    end,
    tick = function(self)
      local t = timer.getTime()
      self._ctrl1:step()
      self._ctrl2:step()
      self._frameStep(29781 - math.max(0, self._phase - 1))
      self._phase = (self._phase + 1) % 3
      return timer.getTime() - t
    end,
    _createMapper = function(self)
      local m = self._cart:getMapper()
      local mapperFound, Mapper_t = pcall(require, "Cart.mappers." .. tostring(m))
      if not (mapperFound) then
        error(("Mapper not found: %d"):format(m))
      end
      return Mapper_t(self)
    end
  }
  _base_0.__index = _base_0
  _class_0 = setmetatable({
    __init = function(self, rom, _frameCallback)
      self._frameCallback = _frameCallback
      jit.off(self.tick)
      self._screen = love.image.newImageData(SCREEN_WIDTH, SCREEN_HEIGHT, 'r8')
      self._cart = Cart(rom)
      self._isPAL = self._cart:getSystem() == 'PAL'
      self._freq = NES[self._cart:getSystem()]
      self._ppu = PPU(self)
      self._apu = APU(self)
      self._mapper = self:_createMapper()
      self._ppu:getMem():setMapper(self._mapper)
      self._bus = Bus(self._mapper, self._ppu, self._apu)
      self._cpu = CPU(self._bus, false)
      self._ctrl1 = Controller(self._apu, 0x4016, {
        a = 'x',
        b = 'c',
        start = 'return',
        select = 'space',
        left = 'left',
        right = 'right',
        up = 'up',
        down = 'down'
      })
      self._ctrl2 = Controller(self._apu, 0x4017)
      self._apu:setCPU(self._cpu)
      self._apu:setController(self._ctrl1, self._ctrl2)
      self._phase = 0
      self._frameCallback = function() end
      do
        local stepP, stepA, stepC = (function()
          local _base_1 = self._ppu
          local _fn_0 = _base_1.step
          return function(...)
            return _fn_0(_base_1, ...)
          end
        end)(), (function()
          local _base_1 = self._apu
          local _fn_0 = _base_1.step
          return function(...)
            return _fn_0(_base_1, ...)
          end
        end)(), (function()
          local _base_1 = self._cpu
          local _fn_0 = _base_1.step
          return function(...)
            return _fn_0(_base_1, ...)
          end
        end)()
        local frameStep
        frameStep = function(steps)
          if steps == 0 then
            return 
          end
          stepP()
          stepC()
          stepA()
          return frameStep(steps - 1)
        end
        self._frameStep = frameStep
      end
    end,
    __base = _base_0,
    __name = "NES"
  }, {
    __index = _base_0,
    __call = function(cls, ...)
      local _self_0 = setmetatable({}, _base_0)
      cls.__init(_self_0, ...)
      return _self_0
    end
  })
  _base_0.__class = _class_0
  NES = _class_0
  return _class_0
end
