local ffi = require('ffi')
local band, bor, bxor, rshift, lshift
do
  local _obj_0 = require('bit')
  band, bor, bxor, rshift, lshift = _obj_0.band, _obj_0.bor, _obj_0.bxor, _obj_0.rshift, _obj_0.lshift
end
local min, max
do
  local _obj_0 = math
  min, max = _obj_0.min, _obj_0.max
end
local ifRendering
ifRendering = function(fn)
  return function(self, mem)
    if not (self.renderingOn > 0) then
      return 
    end
    return fn(self, mem)
  end
end
local incH
do
  local LUT = ffi.new('int16_t[4]', 0x0000, 0x0001, 0x041f, 0x0000)
  incH = ifRendering(function(self)
    self.bgPat[0], self.bgPat[1] = self.bgPat[1], self.bgPat[2]
    self.bgAttr[0], self.bgAttr[1] = self.bgAttr[1], self.bgAttr[2]
    local v = self.vramAddr
    local wrap = lshift(rshift(band(v, 0x1f) + 1, 5), 1)
    self.vramAddr = bxor(v, LUT[wrap]) + LUT[wrap + 1]
  end)
end
local incV = ifRendering(function(self)
  self.bgPat[0], self.bgPat[1] = self.bgPat[1], self.bgPat[2]
  self.bgAttr[0], self.bgAttr[1] = self.bgAttr[1], self.bgAttr[2]
  local v = self.vramAddr
  if band(v, 0x7000) ~= 0x7000 then
    self.vramAddr = v + 0x1000
  else
    v = band(v, 0x0fff)
    local y = rshift(band(v, 0x03e0), 5) + 1
    if y == 30 then
      y, v = 0, bxor(v, 0x0800)
    else
      y = lshift(band(y, 0x1f), 5)
    end
    self.vramAddr = bor(band(v, 0x7c1f), y)
  end
end)
local syncH = ifRendering(function(self)
  self.vramAddr = bor(band(self.vramAddr, 0x7be0), band(self.tempAddr, 0x041f))
end)
local syncV = ifRendering(function(self)
  self.vramAddr = bor(band(self.vramAddr, 0x041f), band(self.tempAddr, 0x7be0))
end)
local _fetchNT
_fetchNT = function(self, mem)
  return mem.readVRAM(0x2000 + band(self.vramAddr, 0xfff))
end
local fetchNT = ifRendering(function(self, mem)
  self.bgTile = _fetchNT(self, mem)
end)
local fetchDummyNT = ifRendering(function(self, mem)
  _fetchNT(self, mem)
  return nil
end)
local fetchAT = ifRendering(function(self, mem)
  local v = self.vramAddr
  local at
  do
    local addr = bor(0x3c0, band(v, 0x0c00), band(rshift(v, 4), 0x38), band(rshift(v, 2), 0x07))
    at = mem.readVRAM(0x2000 + addr)
  end
  local shift = bor(band(rshift(v, 4), 0x04), band(v, 0x02))
  at = band(rshift(at, shift), 0x03)
  self.bgAttr[2] = at
end)
local fetchDummyAT = ifRendering(function(self, mem)
  local v = self.vramAddr
  local addr = bor(0x3c0, band(v, 0x0c00), band(rshift(v, 4), 0x38), band(rshift(v, 2), 0x07))
  mem.readVRAM(0x2000 + addr)
  return nil
end)
local _fetchLoBG
_fetchLoBG = function(self, mem)
  local addr = self.bgTile * 16 + band(rshift(self.vramAddr, 12), 0x07)
  return mem.readCHR(self.bgTable + addr)
end
local fetchLoBG = ifRendering(function(self, mem)
  self.bgPatLo = _fetchLoBG(self, mem)
end)
local fetchDummyLoBG = ifRendering(function(self, mem)
  _fetchLoBG(self, mem)
  return nil
end)
local _fetchHiBG
_fetchHiBG = function(self, mem)
  local addr = self.bgTile * 16 + band(rshift(self.vramAddr, 12), 0x07) + 8
  return mem.readCHR(self.bgTable + addr)
end
local fetchHiBG = ifRendering(function(self, mem)
  self.bgPat[2] = self.bgPatLo * 0x100 + _fetchHiBG(self, mem)
end)
local fetchDummyHiBG = ifRendering(function(self, mem)
  _fetchHiBG(self, mem)
  return nil
end)
local clearOAM = ifRendering(function(self)
  self.oam2[self.OAM2addr] = 0xff
  self.OAM2addr = band(self.OAM2addr + 1, 0x1f)
end)
local resetOAMAddr = ifRendering(function(self)
  self.OAMaddr = 0
end)
local readOAM = ifRendering(function(self)
  if not (self.OAM2addr < 32) then
    return 
  end
  self.oam2[self.OAM2addr] = self.oam[self.OAMaddr]
end)
local writeOAM0 = ifRendering(function(self)
  if not (self.OAMaddr < 255) then
    return 
  end
  do
    local d = self.y - self.oam2[0]
    if d < 0 or d >= self.sprSize then
      self.spr0Active[1] = 0
      return 
    end
  end
  self.spr0Active[1] = 1
  self.oam2[1] = self.oam[self.OAMaddr + 1]
  self.oam2[2] = self.oam[self.OAMaddr + 2]
  self.oam2[3] = self.oam[self.OAMaddr + 3]
  self.OAMaddr = min(255, self.OAMaddr + 4)
  self.OAM2addr = self.OAM2addr + 4
  self.numNextSprites = 1
end)
local writeOAM = ifRendering(function(self)
  if not (self.OAMaddr < 255) then
    return 
  end
  local addr = self.OAM2addr
  local d = self.y - self.oam2[addr]
  if d >= 0 and d < self.sprSize then
    if self.numNextSprites > 7 then
      self.status = bor(self.status, 0x20)
    else
      local inaddr = self.OAMaddr
      self.oam2[addr + 1] = self.oam[inaddr + 1]
      self.oam2[addr + 2] = self.oam[inaddr + 2]
      self.oam2[addr + 3] = self.oam[inaddr + 3]
      self.numNextSprites = self.numNextSprites + 1
      self.OAM2addr = self.OAM2addr + 4
    end
  end
  self.OAMaddr = min(255, self.OAMaddr + 4)
end)
local fetchSPRAT = ifRendering(function(self, mem)
  local r = rshift(self.x - 259, 1)
  local w = lshift(r, 1)
  local spr, oam = self.sprites, self.oam2
  spr[w + 3] = oam[r + 3]
  spr[w + 0] = oam[r + 0]
  spr[w + 1] = oam[r + 1]
  local attr = oam[r + 2]
  spr[w + 2] = attr
  self.sprFlipH = rshift(band(attr, 0x40), 6)
  self.sprFlipV = rshift(band(attr, 0x80), 7)
end)
local reverse
reverse = function(b)
  local r = bor(rshift(band(b, 0xf0), 4), lshift(band(b, 0x0f), 4))
  r = bor(rshift(band(r, 0xcc), 2), lshift(band(r, 0x33), 2))
  return bor(rshift(band(r, 0xaa), 1), lshift(band(r, 0x55), 1))
end
local fetchLoSPR = ifRendering(function(self, mem)
  local spr = self.sprites
  local index = self.x - 262
  local addr
  do
    local tile = spr[index + 1]
    local height = self.sprSize
    local offs
    do
      offs = bxor(max(0, self.y - spr[index]), self.sprFlipV * (height - 1))
      offs = bor(band(offs, 7), lshift(band(offs, 8), 1))
    end
    local long = rshift(height, 4)
    offs = offs + lshift(self.sprTable + band(tile, long), 12)
    addr = offs + rshift(tile, long) * lshift(height, 1)
  end
  self.sprAddr = addr
  local chr = mem.readCHR(addr)
  spr[index + 4] = self.sprFlipH > 0 and reverse(chr) or chr
end)
local fetchHiSPR = ifRendering(function(self, mem)
  local chr = mem.readCHR(self.sprAddr + 8)
  self.sprites[self.x - 264 + 5] = self.sprFlipH > 0 and reverse(chr) or chr
end)
local _renderSprite0
_renderSprite0 = function(self, x, y, bgColor, hitBG, mem)
  local d = 7 - (x - self.sprites[3])
  if d < 0 or d > 7 then
    return bgColor
  end
  local p = band(rshift(self.sprites[4], d), 0x01) + band(rshift(self.sprites[5], d), 0x01) * 2
  if p == 0 then
    return bgColor
  end
  do
    local under255 = (1 - max(0, min(1, x - 254)))
    local hit = hitBG * under255 * self.spr0Active[0]
    self.zeroHit = bor(self.zeroHit, lshift(hit, 8))
  end
  local at = self.sprites[2]
  local pri = band(rshift(at, 5), 0x01) * hitBG
  local col = lshift(band(at, 0x03), 2)
  return bgColor * pri + mem.readPAL(0x3f10 + col + p) * (1 - pri), true
end
local _renderSprite
_renderSprite = function(self, n, x, y, bgColor, hitBG, mem)
  local d = 7 - (x - self.sprites[n + 3])
  if d < 0 or d > 7 then
    return bgColor
  end
  local p = band(rshift(self.sprites[n + 4], d), 0x01) + band(rshift(self.sprites[n + 5], d), 0x01) * 2
  if p == 0 then
    return bgColor
  end
  local at = self.sprites[n + 2]
  local pri = band(rshift(at, 5), 0x01) * hitBG
  local col = lshift(band(at, 0x03), 2)
  return bgColor * pri + mem.readPAL(0x3f10 + col + p) * (1 - pri), true
end
local bgRenderer
bgRenderer = function(self, x, y, mem)
  if x < self.clipBG then
    return mem.readPAL(0x3f00), 0
  end
  local pix = band(x, 7)
  local chrLo, chrHi
  do
    local fx = 7 - self.fineX
    local c1, c2 = self.bgPat[0], self.bgPat[1]
    local lo = bor(lshift(rshift(c1, 8), pix), rshift(rshift(c2, 8), 8 - pix))
    local hi = bor(lshift(band(c1, 0xff), pix), rshift(band(c2, 0xff), 8 - pix))
    local m = lshift(1, fx)
    chrLo, chrHi = rshift(band(lo, m), fx), rshift(band(hi, m), fx)
  end
  local hitBG = bor(chrLo, chrHi)
  local palLo = chrHi * 2 + chrLo
  local palHi = hitBG * 4 * self.bgAttr[band(rshift(self.fineX + pix, 3), 0x01)]
  return mem.readPAL(0x3f00 + palHi + palLo), hitBG
end
local render
render = function(self, mem)
  local x, y = self.x, self.y
  local res, hitBG = bgRenderer(self, x, y, mem)
  if self.numSprites > 0 and x >= self.clipSpr then
    local done
    res, done = _renderSprite0(self, x, y, res, hitBG, mem)
    local i = 1
    while not done and i < self.numSprites do
      res, done = _renderSprite(self, i * 8, x, y, res, hitBG, mem)
      i = i + 1
    end
  end
  self.frameBuffer[x + y * SCREEN_WIDTH] = res
  self.zeroHit = rshift(self.zeroHit, 1)
  self.status = bor(self.status, band(self.zeroHit, 0x40))
end
local renderBG
renderBG = function(self, mem)
  local x, y = self.x, self.y
  self.frameBuffer[x + y * SCREEN_WIDTH] = bgRenderer(self, x, y, mem)
end
local scanlineStart
scanlineStart = function(self)
  self.numSprites, self.numNextSprites = self.numNextSprites, 0
  self.OAM2addr = 0
  self.spr0Active[0], self.spr0Active[1] = self.spr0Active[1], 0
end
local idle
idle = function(self) end
local skipOnOdd
skipOnOdd = function(self)
  self.x = self.x + min(1, self.oddFrame * self.renderingOn)
end
local frameStart
frameStart = function(self, mem)
  self.lastStatusRead = 0
end
local enterVBL
enterVBL = function(self)
  if band(self.status, 0x80) == 0 then
    return self:present()
  end
  if self.cycleCount - self.lastStatusRead < 2 then
    self.status = bor(self.status, 0x80)
    return self:present()
  end
  self.status, self.nmiFired = bor(self.status, 0x80), 1
  self:nmi()
  return self:present()
end
local setVBL
setVBL = function(self)
  if self.lastStatusRead < self.cycleCount then
    self.status = bor(self.status, 0x80)
  end
end
local clearVBL
clearVBL = function(self)
  self.status = band(self.status, 0x7f)
end
local clearSPRFlags
clearSPRFlags = function(self)
  self.status = band(self.status, 0x9f)
end
local vblank
vblank = function(self)
  if not (self.nmiFired == 0 and band(self.status, 0x80) + band(self.ctrl, 0x80) == 0x100) then
    return 
  end
  self.nmiFired = 1
  return self:nmi()
end
local leaveVBL
leaveVBL = function(self)
  self.status = band(self.status, 0x1f)
  self.nmiFired = 0
end
return {
  idle = idle,
  incH = incH,
  incV = incV,
  syncH = syncH,
  syncV = syncV,
  fetchNT = fetchNT,
  fetchDummyNT = fetchDummyNT,
  fetchAT = fetchAT,
  fetchDummyAT = fetchDummyAT,
  fetchLoBG = fetchLoBG,
  fetchDummyLoBG = fetchDummyLoBG,
  fetchHiBG = fetchHiBG,
  fetchDummyHiBG = fetchDummyHiBG,
  render = render,
  renderBG = renderBG,
  fetchLoSPR = fetchLoSPR,
  fetchHiSPR = fetchHiSPR,
  fetchSPRAT = fetchSPRAT,
  clearOAM = clearOAM,
  resetOAMAddr = resetOAMAddr,
  readOAM = readOAM,
  writeOAM = writeOAM,
  writeOAM0 = writeOAM0,
  enterVBL = enterVBL,
  leaveVBL = leaveVBL,
  setVBL = setVBL,
  clearVBL = clearVBL,
  clearSPRFlags = clearSPRFlags,
  vblank = vblank,
  frameStart = frameStart,
  scanlineStart = scanlineStart,
  skipOnOdd = skipOnOdd
}
